"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@morjs/utils");
const utils_2 = require("../../generate/utils");
class DataBinding {
    constructor(express) {
        this.type = 'DataBindingNode';
        this._hasBinding = false;
        this._bindingVars = new Set();
        this._bindingExpression = undefined;
        this.expParts = [];
        this._express = express;
        this._bindingExpression = this.parse(express);
    }
    get express() {
        return this._express;
    }
    /**
     * 是否有绑定
     */
    get hasBinding() {
        return this._hasBinding;
    }
    get bindingVars() {
        return this._bindingVars;
    }
    get bindingExpression() {
        return this._bindingExpression;
    }
    parse(text) {
        if (!text)
            return;
        text = text.trim().replace(/\{\{\{/, '{{ {');
        let exp = text;
        // const expParts: Array<ExpPartInfo> = [];
        // eslint-disable-next-line no-constant-condition
        while (true) {
            if (exp.length === 0)
                break;
            // 整体是从右往左扫描。如果从左往右扫描会可能会有bug，比如: {{ {a:{a:1}} }}
            const idx1 = exp.lastIndexOf('{{');
            const idx2 = exp.lastIndexOf('}}');
            if (idx1 === -1 || idx2 === -1 || idx2 < idx1) {
                this.expParts.push({ code: `${exp}` });
                break;
            }
            // 如果idx2 非字符串最后位置,那么说明}} 后面又一段普通字符串
            if (idx2 < exp.length - 2) {
                this.expParts.push({ code: `${exp.substring(idx2 + 2)}` });
            }
            // 提取表达式
            const bindExp = exp.substring(idx1 + 2, idx2).trim();
            this.expParts.push({ code: bindExp, binding: true });
            exp = exp.substring(0, idx1);
        }
        this.expParts.reverse();
        // 提取表达式，以及 对 对象 的提取
        this.expParts.forEach((p) => {
            let code = p.code;
            if (p.binding) {
                let res = this.parseExp(code);
                // 如果变量提取失败，那么默认为是对象表达式
                if (!res.isOK) {
                    code = `{ ${code} }`;
                    res = this.parseExp(code);
                    // 再次提取变量
                    if (!res.isOK) {
                        utils_1.logger.error(`绑定表达式: ${code.trim()} 不符合 JS 表达式规范`);
                    }
                }
                if (res.code) {
                    code = res.code;
                }
                const isComplexExp = /\?|\+|-|\*|\/|&&|\|\|/.test(code);
                if (isComplexExp) {
                    code = `(${code})`;
                }
                this._hasBinding = true;
            }
            else {
                code = code.replace(/[\r\n]/g, '');
                if (code.indexOf('{') >= 0 || code.indexOf('}') >= 0) {
                    this._hasBinding = true;
                    code = `\`${code}\``;
                }
                else {
                    // 普通字符串 添加 ""
                    code = `"${code.replace(/"/g, '\\"')}"`; // 替换引号，防止会出现bug
                }
            }
            p.code = code;
        });
        if (this._hasBinding) {
            return this.expParts.map((p) => p.code).join(' + ');
        }
        return this.express;
    }
    // 提取变量
    parseExp(exp) {
        const res = { isOK: false, code: undefined };
        if (!exp)
            return res;
        try {
            const ast = utils_1.babelParser.parse(`var _=(${exp})`);
            res.isOK = true;
            utils_1.babelTraverse.default(ast, {
                ReferencedIdentifier: (path) => {
                    // 检索出引用标识符
                    this._bindingVars.add(path.node.name);
                },
                SequenceExpression() {
                    res.isOK = false;
                }
            });
        }
        catch (e) {
            res.isOK = false;
        }
        return res;
    }
    getExpressionAst(options) {
        if (this.hasBinding) {
            let ast = this.expressionAst(`(${this.bindingExpression})`);
            if (options && options.forceObject) {
                if (ast && !utils_1.babelTypes.isObjectExpression(ast)) {
                    ast = this.expressionAst(`_={${this.bindingExpression}}`);
                    return ast && ast.right;
                }
            }
            return ast;
        }
        return null;
    }
    expressionAst(bindingExpression) {
        try {
            const exp = utils_1.babelParser.parse(bindingExpression).program.body[0];
            if (utils_1.babelTypes.isExpressionStatement(exp)) {
                return exp.expression;
            }
            else {
                utils_1.logger.error('绑定表达式错误: ' + this.express);
                return null;
            }
        }
        catch (e) {
            return null;
        }
    }
    getExpressionAstForText() {
        if (this.hasBinding) {
            const bindingExpression = `${this.expParts
                .map((p) => {
                if (p.binding) {
                    return `$rm.getString(${(0, utils_2.getOptionalChainCode)(p.code)})`;
                }
                else {
                    return p.code;
                }
            })
                .join(' + ')}`;
            return this.expressionAst(bindingExpression);
        }
        return null;
    }
}
exports.default = DataBinding;
//# sourceMappingURL=index.js.map