"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@morjs/utils");
const utils_2 = require("../../../../utils");
const option_1 = require("../../../option");
const index_1 = __importDefault(require("../attributes/index"));
const helper_databinding_1 = require("../helper-databinding");
const utils_3 = require("../utils");
const index_2 = __importDefault(require("./index"));
function default_1(node, context) {
    const externalName = getExternalComponent(context.options, node.name);
    //  属性
    const atts = externalName
        ? node.attributes.map((att) => (0, index_1.default)(node, att, context))
        : node.attributes
            .filter((t) => t.type !== 'EventAttributeNode')
            .map((att) => (0, index_1.default)(node, att, context));
    const children = [];
    node.children.forEach((el) => {
        children.push(utils_1.babelTypes.jsxText('\r\n'));
        children.push((0, index_2.default)(el, context));
        children.push(utils_1.babelTypes.jsxText('\r\n'));
    });
    const globalName = (0, option_1.globalComponentName)(node.name, context.options);
    if (globalName) {
        // 转换小程序组件名称。系统组件的名称优先级大于 自定义的
        node.name = externalName || globalName;
        // atts.push(t.jsxAttribute(t.jsxIdentifier('tiga_node_id'), t.stringLiteral(node.nodeId)))
    }
    const eventAtts = node.attributes.filter((t) => t.type === 'EventAttributeNode');
    const classKey = externalName ? 'className' : 'class';
    if (eventAtts.length > 0) {
        if (globalName) {
            const arraryExp = utils_1.babelTypes.arrayExpression(eventAtts.map((att) => {
                context.mergeDatabindingVars(att.value);
                return utils_1.babelTypes.objectExpression([
                    utils_1.babelTypes.objectProperty(utils_1.babelTypes.identifier('name'), utils_1.babelTypes.stringLiteral(att.name.toLowerCase())),
                    utils_1.babelTypes.objectProperty(utils_1.babelTypes.identifier('event'), (0, helper_databinding_1.databindingForValue)(att.value)),
                    utils_1.babelTypes.objectProperty(utils_1.babelTypes.identifier('catch'), utils_1.babelTypes.booleanLiteral(att.isCatch))
                ]);
            }));
            // for循环节点的nodeId = nodeId + index
            const isIndexDefinedExpression = utils_1.babelTypes.binaryExpression('!==', utils_1.babelTypes.identifier('typeof index'), utils_1.babelTypes.stringLiteral('undefined'));
            const indexConditionalExpression = utils_1.babelTypes.conditionalExpression(isIndexDefinedExpression, utils_1.babelTypes.identifier('index'), utils_1.babelTypes.stringLiteral(''));
            if (!externalName) {
                children.push(utils_1.babelTypes.jsxExpressionContainer(utils_1.babelTypes.callExpression(utils_1.babelTypes.memberExpression(utils_1.babelTypes.identifier('$rm'), utils_1.babelTypes.identifier('registEvents')), [
                    arraryExp,
                    utils_1.babelTypes.identifier('$data'),
                    utils_1.babelTypes.binaryExpression('+', utils_1.babelTypes.stringLiteral(node.nodeId), indexConditionalExpression)
                ])));
            }
            // 添加class node.nodeId。 当然这个也可以在后续的功能迭代中直接作为公共设置。
            const classAtt = atts.filter((i) => i.name.name === 'class').pop();
            const compIdExpression = utils_1.babelTypes.binaryExpression('+', utils_1.babelTypes.stringLiteral(` comp-id-`), utils_1.babelTypes.identifier('$id'));
            const eventClassNameExp = utils_1.babelTypes.binaryExpression('+', utils_1.babelTypes.binaryExpression('+', utils_1.babelTypes.stringLiteral(` ${node.nodeId}`), indexConditionalExpression), compIdExpression);
            if (classAtt) {
                classAtt.name = utils_1.babelTypes.jsxIdentifier(classKey);
                const attValue = classAtt.value;
                attValue.expression = utils_1.babelTypes.binaryExpression('+', (0, utils_3.addBracketToIdentifier)((0, utils_3.getOptionalChainExpression)(attValue.expression)), eventClassNameExp);
            }
            else {
                atts.push(utils_1.babelTypes.jsxAttribute(utils_1.babelTypes.jsxIdentifier(classKey), utils_1.babelTypes.jsxExpressionContainer(eventClassNameExp)));
            }
            context.dataBindingVars.add('$id');
        }
        else {
            atts.push(...eventAtts.map((att) => (0, index_1.default)(node, att, context)));
        }
    }
    else {
        const classAtt = atts.filter((i) => i.name.name === 'class').pop();
        if (classAtt) {
            classAtt.name = utils_1.babelTypes.jsxIdentifier(classKey);
            const attValue = classAtt.value;
            attValue.expression = (0, utils_3.getOptionalChainExpression)(attValue.expression);
        }
    }
    const enableStyleScope = (0, utils_2.isEnableStyleScope)(context.options); // 是否启用样式隔离
    if (enableStyleScope) {
        const styleScopeHash = (0, option_1.randomHash)(context.options);
        const classAtt = atts.filter((i) => i.name.name === 'class').pop();
        if (classAtt) {
            const attValue = classAtt.value;
            attValue.expression = utils_1.babelTypes.binaryExpression('+', (0, utils_3.addBracketToIdentifier)(attValue.expression), utils_1.babelTypes.stringLiteral(` ${styleScopeHash}`));
        }
        else {
            atts.push(utils_1.babelTypes.jsxAttribute(utils_1.babelTypes.jsxIdentifier(classKey), utils_1.babelTypes.jsxExpressionContainer(utils_1.babelTypes.stringLiteral(` ${styleScopeHash}`))));
        }
    }
    return utils_1.babelTypes.jsxElement(utils_1.babelTypes.jsxOpeningElement(utils_1.babelTypes.jsxIdentifier(node.name), atts, children.length === 0), utils_1.babelTypes.jsxClosingElement(utils_1.babelTypes.jsxIdentifier(node.name)), children, null);
}
exports.default = default_1;
function getExternalComponent(options, key) {
    const { externalComponents = {} } = options;
    return externalComponents[key] ? (0, utils_3.getComponentName)(key) : '';
}
//# sourceMappingURL=ElementNode.js.map