"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserConfigSchema = exports.fileNameConfig = exports.generateEntryRoot = exports.WEB_COMPILER_LOADERS = exports.WEB_RUNTIMES = exports.CURRENT_NODE_MODULES = exports.RUNTIME_NPM_NAME = exports.ASSETS_REGEXP = exports.templateSingleTagClosingType = exports.projectConfigFiles = exports.defaultOutputDir = exports.defaultConditionalFileExt = exports.resolveMainFields = exports.globalObject = exports.targetDescription = exports.target = void 0;
const utils_1 = require("@morjs/utils");
const path_1 = __importDefault(require("path"));
/**
 * 以下为 mor 编译插件需要
 */
exports.target = 'web';
exports.targetDescription = 'Web 应用';
exports.globalObject = 'my';
exports.resolveMainFields = ['miniforweb', 'module', 'main'];
exports.defaultConditionalFileExt = '.web';
exports.defaultOutputDir = 'dist/web';
exports.projectConfigFiles = [];
// 转 web 自闭合标签的闭合方式设置为 tag
// 行为如下: <image /> => <image></image>
// 原因为: sax 解析时最后一个属性如果无值会导致闭合错误抛错 <image someProp />
exports.templateSingleTagClosingType = 'tag';
/**
 * 静态资源支持
 */
exports.ASSETS_REGEXP = /\.(jpg|jpeg|png|svg|bmp|ico|gif|webp|otf|ttf|woff|woff2|eot|cer|ogg|aac|mp4|wav|mp3)$/i;
exports.RUNTIME_NPM_NAME = '@morjs/runtime-web';
let RUNTIME_NPM_SEGMENTS;
try {
    RUNTIME_NPM_SEGMENTS = require.resolve(exports.RUNTIME_NPM_NAME).split('node_modules');
    // 移除最后的文件，则前面的路径为 node_modules 的路径
    RUNTIME_NPM_SEGMENTS.pop();
    if (RUNTIME_NPM_SEGMENTS.length > 0)
        RUNTIME_NPM_SEGMENTS.push('');
}
catch (error) {
    utils_1.logger.debug(`未能定位到 ${exports.RUNTIME_NPM_NAME} 的实际位置`);
}
/**
 * 当前 npm 所包含的 node_modules 目录
 * 以及 RUNTIME_NPM_NAME 所在的 node_modules 目录
 */
exports.CURRENT_NODE_MODULES = [
    path_1.default.resolve(__dirname, '../node_modules')
].concat(RUNTIME_NPM_SEGMENTS.length ? RUNTIME_NPM_SEGMENTS.join('node_modules') : []);
exports.WEB_RUNTIMES = {
    runtime: `${exports.RUNTIME_NPM_NAME}/lib/runtime`,
    api: `${exports.RUNTIME_NPM_NAME}/lib/api`,
    components: `${exports.RUNTIME_NPM_NAME}/lib/components`,
    router: `${exports.RUNTIME_NPM_NAME}/lib/router`,
    config: `${exports.RUNTIME_NPM_NAME}/lib/components/config`
};
exports.WEB_COMPILER_LOADERS = {
    template: path_1.default.resolve(__dirname, './loaders/template'),
    script: path_1.default.resolve(__dirname, './loaders/script'),
    style: path_1.default.resolve(__dirname, './loaders/style')
};
/**
 * 生成虚拟的 entry 根目录
 */
function generateEntryRoot(srcPath) {
    return path_1.default.join(srcPath, `${utils_1.mor.name}-entries`);
}
exports.generateEntryRoot = generateEntryRoot;
/**
 * 获取 webpack 文件名称配置
 */
function fileNameConfig(mode, compileMode) {
    return mode === 'production' && compileMode === 'bundle'
        ? '[name].[contenthash:6]'
        : '[name]';
}
exports.fileNameConfig = fileNameConfig;
/**
 * Web 相关用户配置
 */
exports.UserConfigSchema = utils_1.zod.object({
    /**
     * web 编译相关配置
     */
    web: utils_1.zod
        .object({
        // 配置小程序组件名映射表
        globalComponentsConfig: utils_1.zod.record(utils_1.zod.string()).optional(),
        // 是否使用 rpx2rem 插件对 rpx 单位进行转换
        needRpx: utils_1.zod.boolean().optional(),
        // 是否需要样式隔离，如果开启该功能，在编译时会给样式加上唯一 hash 值，用于多页面解决样式冲突问题
        styleScope: utils_1.zod.boolean().optional(),
        // rpx 方案的 root value。默认是 32
        rpxRootValue: utils_1.zod.number().optional(),
        // 默认编译出来的样式会以 rem 为单位，
        // 配合 runtime 层提供的 setRootFontSizeForRem 方法可以实现移动端的响应式。
        // 如果不想将样式单位编译成 rem 可以配置该对象，
        // 对象中包含一个参数 ratio 用于指定 rpx 2 px 的转换比例。
        // 如 ratio 为 1， 那么 1rpx = 1px， 如果配置 ratio 为 2， 那么 1rpx = 0.5px
        usePx: utils_1.zod.object({ ratio: utils_1.zod.number() }).optional(),
        // 编译过程中可以引入第三方插件，目前仅支持在 js 文件打包过程中引入 babel 插件
        // compilerPlugins: z.object({ js: z.array(z.any()).optional() }).optional(),
        // extensions 配置，用于设置接口扩展
        // 可设置多个扩展路径，如 ['extension1', 'extension2', ['extension3', {}]]
        extensions: utils_1.zod
            .array(utils_1.zod.string().or(utils_1.zod.tuple([utils_1.zod.string(), utils_1.zod.record(utils_1.zod.any()).optional()])))
            .optional(),
        // 配置可开启页面自动响应式（以375尺寸为准）
        responsiveRootFontSize: utils_1.zod.number().optional(),
        // 是否需要在导航栏展示返回按钮
        showBack: utils_1.zod
            .boolean()
            .or(utils_1.zod.array(utils_1.zod.string()))
            .or(utils_1.zod.function().args(utils_1.zod.string()).returns(utils_1.zod.boolean()))
            .default(false),
        // 是否需要导航栏
        showHeader: utils_1.zod
            .boolean()
            .or(utils_1.zod.array(utils_1.zod.string()))
            .or(utils_1.zod.function().args(utils_1.zod.string()).returns(utils_1.zod.boolean()))
            .default(true),
        // 产物通用路径
        publicPath: utils_1.zod.string().default('/'),
        // 开发服务器配置
        devServer: utils_1.zod.record(utils_1.zod.any()).optional(),
        // html-webpack-plugin 插件 配置
        // 支持配置多个 html 页面
        htmlOptions: utils_1.zod.array(utils_1.zod.record(utils_1.zod.any())).or(utils_1.zod.record(utils_1.zod.any())).optional(),
        // mini-css-extract-plugin 插件配置
        miniCssExtractOptions: utils_1.zod.record(utils_1.zod.any()).optional(),
        // 行内资源大小配置, 默认为 8k
        inlineAssetLimit: utils_1.zod.number().default(8192),
        // 是否自动生成 entries, 默认均开启
        // 如关闭, 则需要自行指定 webpack 配置的入口文件
        autoGenerateEntries: utils_1.zod.boolean().default(true),
        // 入口文件配置, 仅在 autoGenerateEntries 为 false 的情况下生效
        // 用于手动配置 webpack 入口文件地址
        // 若手动配置则需要用户自行组装页面及引用
        entry: utils_1.zod.any().optional(),
        // 是否输出中间产物
        emitIntermediateAssets: utils_1.zod.boolean().default(false),
        // 支持用户自定义运行时的一些行为，该字段将作为 $MOR_APP_CONFIG 写入到 window 对象中
        // 供运行时获取及消费
        appConfig: utils_1.zod
            .object({
            /**
             * 用于将样式文件中 @import 样式的内容写入到样式文件中
             */
            enableCombineImportStyles: utils_1.zod.boolean().default(false),
            /**
             * 用于传递业务对某个组件的特殊配置, 比如 map 组件传递 key
             */
            components: utils_1.zod.record(utils_1.zod.any()).optional(),
            /* 用于传递业务对某个 api 的特殊配置，比如 clipboard 切换实现方式 */
            apis: utils_1.zod.record(utils_1.zod.any()).optional(),
            /**
             * 用于自定义 URL 中 query 部分的 tabBarKey 的实际名称，该 key 作用于 URL 的
             * query，并将拿到到值，用于获取 app.json 中的 tabbar 内容，以此实现通过 URL 参数
             * 切换不同 tabbar 的功能
             */
            customTabBarKey: utils_1.zod.string().default('tabBarKey'),
            /**
             * 用于配置是否允许覆盖全局对象中的方法
             */
            apiNoConflict: utils_1.zod.boolean().optional(),
            // 用于支持在代码中动态设置 header 的展示与隐藏
            pageHeaderConfig: utils_1.zod
                .object({
                showHeader: utils_1.zod.boolean().optional(),
                showBack: utils_1.zod.boolean().optional()
            })
                .optional(),
            // 导航栏信息设置(支持 字符串/数字/函数)
            nav: utils_1.zod
                .object({
                statusBarHeight: utils_1.zod.number().or(utils_1.zod.string()).optional(),
                titleBarHeight: utils_1.zod.number().or(utils_1.zod.string()).optional()
            })
                .optional()
        })
            .default({})
    })
        .default({})
});
//# sourceMappingURL=constants.js.map