"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScriptCompatiblePlugin = void 0;
const utils_1 = require("@morjs/utils");
const path_1 = __importDefault(require("path"));
const EXPORT_DEFAULT_REGEXP = /export +default/;
/**
 * Script 兼容插件
 * 1. 移除页面或组件 脚本文件中的 export default，原因 转 web 需要使用这种导出方式
 */
class ScriptCompatiblePlugin {
    constructor() {
        this.name = 'MorWebScriptCompatiblePlugin';
    }
    apply(runner) {
        runner.hooks.preprocessorParser.tap(this.name, (content, context, options) => {
            if (options.fileInfo.entryFileType === utils_1.EntryFileType.script &&
                (options.fileInfo.entryType === utils_1.EntryType.page ||
                    options.fileInfo.entryType === utils_1.EntryType.component ||
                    options.fileInfo.entryType === utils_1.EntryType.npmComponent)) {
                // NOTE: 这里的移除方式相对比较粗暴，后续可以考虑换位 ts 插件来处理
                if (EXPORT_DEFAULT_REGEXP.test(content)) {
                    const filePath = path_1.default.relative(runner.getCwd(), options.fileInfo.path);
                    utils_1.logger.warnOnce(`已自动移除组件或页面的 export default 导出\n` +
                        `原因: 该导出方式为 MorJS 转 Web 页面或组件的标准导出方式\n` +
                        `文件: ${filePath}`);
                    return content.replace(EXPORT_DEFAULT_REGEXP, '');
                }
            }
            return content;
        });
    }
}
exports.ScriptCompatiblePlugin = ScriptCompatiblePlugin;
//# sourceMappingURL=scriptCompatiblePlugin.js.map